<?php


namespace Revolut\Payment\Controller\Process;

use Magento\Quote\Model\QuoteFactory;
use Magento\Framework\App\Request\Http;
use Magento\Framework\App\ActionInterface;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Framework\Controller\Result\RedirectFactory;

use Revolut\Payment\Model\Helper\Logger;
use Revolut\Payment\Model\RevolutOrder;
use Revolut\Payment\Model\RevolutPaymentInformationManagement;

/**
 * Class Payment
 */
class Payment implements ActionInterface
{
    /**
     * @var QuoteFactory
     */
    protected $quoteFactory;

    /**
     * @var Logger
     */
    public $logger;
    
    /**
     * @var Http
     */
    public $request;

    /**
     * @var RevolutOrder
     */
    protected $revolutOrder;

    /**
     * @var RedirectFactory
     */
    protected $redirectFactory;

    /**
     * @var StoreManagerInterface
     */
    protected $storeManager;
    
    /**
     * @var RevolutPaymentInformationManagement
     */
    protected $revolutPaymentInformationManagement;
    
    /**
     * Payment constructor.
     *
     * @param Http $request
     * @param Logger $logger
     * @param QuoteFactory $quoteFactory
     * @param RevolutOrder $revolutOrder
     * @param RedirectFactory $redirectFactory
     * @param StoreManagerInterface $storeManager
     * @param RevolutPaymentInformationManagement $revolutPaymentInformationManagement
     */
    public function __construct(
        Http $request,
        Logger $logger,
        QuoteFactory $quoteFactory,
        RevolutOrder $revolutOrder,
        RedirectFactory $redirectFactory,
        StoreManagerInterface $storeManager,
        RevolutPaymentInformationManagement $revolutPaymentInformationManagement
    ) {
        $this->request = $request;
        $this->logger = $logger;
        $this->quoteFactory = $quoteFactory;
        $this->revolutOrder = $revolutOrder;
        $this->redirectFactory = $redirectFactory;
        $this->storeManager = $storeManager;
        $this->revolutPaymentInformationManagement = $revolutPaymentInformationManagement;
    }

    public function execute()
    {
        try {
            $publicId = $this->request->getParam('_rp_oid');

            if (empty($publicId)) {
                throw new \Exception("public id is missing");
            }
                
            $revolutOrder = $this->revolutOrder->retrieveBy('public_id', $publicId);

            if (empty($revolutOrder->getQuoteId())) {
                throw new \Exception("Can not find quote for public id: " . $publicId);
            }

            $quote = $this->quoteFactory->create()->load($revolutOrder->getQuoteId());
            $storeId = $quote->getStoreId();

            if (empty($storeId)) {
                throw new \Exception("Can not find store for public id: " . $publicId);
            }

            $store = $this->storeManager->getStore($storeId);

            if (!empty($this->request->getParam('_rp_fr'))) {
                return $this->redirectFactory->create()->setUrl($store->getBaseUrl() . 'checkout?_rp_fr=' . $this->request->getParam('_rp_fr') . '#payment');
            }

            $result = $this->revolutPaymentInformationManagement->placeOrder($this->request->getParam('_rp_oid')); // @phpstan-ignore-line
            
            if (!empty($result['orderId'])) {
                return $this->redirectFactory->create()->setUrl($store->getBaseUrl() . 'checkout/onepage/success');
            }

            throw new \Exception("Can not place order : " . \json_encode($result));
        } catch (\Exception $e) {
            $this->logger->debug('An error occurred while processing the Magento order: ' . $e->getMessage());
        }
        return $this->redirectFactory->create()->setUrl('/checkout?_rp_fr=Something went wrong. Payment will be canceled#payment');
    }
}
