<?php

namespace Revolut\Payment\Model\Api;

use http\Env\Request as Req;
use Revolut\Payment\Model\Helper\Logger;
use Revolut\Payment\Model\Helper\ConstantValue;
use Revolut\Payment\Gateway\Config\Config as ConfigHelper;
use Magento\Framework\Exception\LocalizedException;
use Revolut\Payment\Gateway\Helper\AmountProvider;
use Revolut\Payment\Model\Helper\Api\RevolutOrderApi;
use Revolut\Payment\Model\RevolutOrderFactory;
use Revolut\Payment\Api\FastCheckoutManagementInterface;
use Revolut\Payment\Gateway\Config\RevolutPaymentRequest\Config as RevolutPaymentRequestConfig;

use Magento\Framework\Registry;
use Magento\Tax\Helper\Data as TaxHelper;
use Magento\Framework\Json\Helper\Data as JsonHelper;
use Magento\Checkout\Model\Session as CheckoutSession;
use Magento\Customer\Model\Session as CustomerSession;
use Magento\Catalog\Model\ProductFactory;
use Magento\Framework\Webapi\Rest\Request;
use Magento\Quote\Api\ShipmentEstimationInterface;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Checkout\Model\Cart;
use Revolut\Payment\Model\ChangeQuoteControl;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Quote\Api\CartRepositoryInterface;
use Magento\Directory\Model\CountryFactory;
use Magento\Framework\Webapi\ServiceInputProcessor;
use Magento\Checkout\Api\Data\ShippingInformationInterfaceFactory;
use Magento\Checkout\Api\ShippingInformationManagementInterface;

/**
 * Class FastCheckoutManagement
 */
class FastCheckoutManagement implements FastCheckoutManagementInterface
{
    /**
     * @var Registry
     */
    protected $registry;

    /**
     * @var RevolutOrderApi
     */
    protected $revolutOrderApi;
    
    /**
     * @var AmountProvider
     */
    protected $amountProvider;
    
    /**
     * @var JsonHelper
     */
    protected $jsonHelper;

    /**
     * @var ConfigHelper
     */
    protected $configHelper;

    /**
     * @var CheckoutSession
     */
    protected $checkoutSession;

    /**
     * @var ProductFactory
     */
    protected $productFactory;

    /**
     * @var Request
     */
    protected $request;

    /**
     * @var RevolutOrderFactory
     */
    protected $revolutOrderFactory;

    /**
     * @var Logger
     */
    protected $logger;

    /**
     * @var TaxHelper
     */
    private $taxHelper;
    
    /**
     * @var ShipmentEstimationInterface
     */
    private $shipmentEstimation;

    /**
     * @var StoreManagerInterface
     */
    private $storeManager;

    /**
     * @var ProductRepositoryInterface
     */
    private $productRepository;

    /**
     * @var CartRepositoryInterface
     */
    private $quoteRepository;

    /**
     * @var Cart
     */
    private $cart;
    
    /**
     * @var CountryFactory
     */
    private $countryFactory;

    /**
     * @var ServiceInputProcessor
     */
    private $inputProcessor;

    /**
     * @var CustomerSession
     */
    private $customerSession;
    
    /**
     * @var ShippingInformationInterfaceFactory
     */
    private $shippingInformationFactory;

    /**
     * @var ShippingInformationManagementInterface
     */
    private $shippingInformationManagement;
    
    /**
     * @var RevolutPaymentRequestConfig
     */
    private $paymentRequestConfigHelper;

    /**
     * @var ChangeQuoteControl
     */
    private $changeQuoteControl;

    public function __construct(
        Registry $registry,
        JsonHelper $jsonHelper,
        ConfigHelper $configHelper,
        CheckoutSession $checkoutSession,
        ProductFactory $productFactory,
        Request $request,
        Logger $logger,
        RevolutOrderFactory $revolutOrderFactory,
        TaxHelper $taxHelper,
        Cart $cart,
        CustomerSession $customerSession,
        ShipmentEstimationInterface $shipmentEstimation,
        StoreManagerInterface  $storeManager,
        ProductRepositoryInterface $productRepository,
        CartRepositoryInterface $quoteRepository,
        CountryFactory $countryFactory,
        ServiceInputProcessor $inputProcessor,
        ShippingInformationInterfaceFactory $shippingInformationFactory,
        ShippingInformationManagementInterface $shippingInformationManagement,
        AmountProvider $amountProvider,
        RevolutOrderApi $revolutOrderApi,
        RevolutPaymentRequestConfig $paymentRequestConfigHelper,
        ChangeQuoteControl $changeQuoteControl
    ) {
        $this->registry = $registry;
        $this->jsonHelper = $jsonHelper;
        $this->configHelper = $configHelper;
        $this->checkoutSession = $checkoutSession;
        $this->productFactory = $productFactory;
        $this->revolutOrderFactory = $revolutOrderFactory;
        $this->logger = $logger;
        $this->taxHelper = $taxHelper;
        $this->cart = $cart;
        $this->request = $request;
        $this->customerSession = $customerSession;
        $this->shipmentEstimation = $shipmentEstimation;
        $this->storeManager = $storeManager;
        $this->productRepository = $productRepository;
        $this->quoteRepository = $quoteRepository;
        $this->countryFactory = $countryFactory;
        $this->inputProcessor = $inputProcessor;
        $this->shippingInformationFactory = $shippingInformationFactory;
        $this->shippingInformationManagement = $shippingInformationManagement;
        $this->amountProvider = $amountProvider;
        $this->revolutOrderApi = $revolutOrderApi;
        $this->paymentRequestConfigHelper = $paymentRequestConfigHelper;
        $this->changeQuoteControl = $changeQuoteControl;
    }

     /**
      * @return mixed
      */
    public function getThemeConfigs()
    {
        return \json_encode($this->paymentRequestConfigHelper->getPaymentRequestButtonStyleConfigs());
    }

    /**
     * @return string
     */
    public function getRevolutSdk()
    {
        $mode = $this->configHelper->getModeName($this->storeManager->getStore()->getId());
        return ConstantValue::REVOLUT_SDK_URLS[$mode];
    }

    /**
     * @return int
     */
    public function getProductId()
    {
        $product = $this->registry->registry('product');
        if ($product) {
            return $product->getId();
        }
        return 0;
    }

    /**
     * @param  string $location
     * @return bool
     */
    public function isActiveLocation($location)
    {
        $storeId = $this->storeManager->getStore()->getId();
        return $this->configHelper->getWebhookSetupResult($storeId) &&
               $this->configHelper->isLiveMode($storeId) &&
               $this->paymentRequestConfigHelper->isActiveLocation($location, $storeId);
    }

    /**
     * @return string
     */
    public function getParams()
    {
        $location = $this->request->getParam('location');
        $storeCode = $this->storeManager->getStore()->getCode();
        $productId = $this->getProductId();
        $revolutSdk = $this->getRevolutSdk();
        $methodCode = \Revolut\Payment\Model\Ui\ConfigProvider::REVOLUT_PAYMENT_REQUEST_CODE;
        $themeConfigs = $this->getThemeConfigs();

        return $this->jsonHelper->jsonEncode([
            'storeCode' => $storeCode,
            'productId' => $productId,
            'revolutSdk' => $revolutSdk,
            'methodCode' => $methodCode,
            'themeConfigs' => $themeConfigs,
            'location' => $location,
            'isActiveLocation' => $this->isActiveLocation($location), // @phpstan-ignore-line
        ]);
    }

    /**
     * @return string
     */
    public function addToCart()
    {
        $request = $this->request->getBodyParams();
        $params = [];
        parse_str($request['request'], $params);

        $productId = (int)$params['product'];
        $related = $params['related_product'];

        $quote = $this->cart->getQuote();

        try {
            $storeId = $this->storeManager->getStore()->getId();
            $product = $this->productRepository->getById($productId, false, $storeId);

            $isUpdated = false;
            foreach ($quote->getAllItems() as $item) {
                if ($item->getProductId() == $productId) {
                    $item = $this->cart->updateItem($item->getId(), $params);
                    if ($item->getHasError()) { // @phpstan-ignore-line
                        throw new LocalizedException(__($item->getMessage())); // @phpstan-ignore-line
                    }

                    $isUpdated = true;
                    break;
                }
            }

            if (!$isUpdated) {
                // @phpstan-ignore-next-line
                $item = $this->cart->addProduct($product, $params);
                if ($item->getHasError()) {
                    throw new LocalizedException(__($item->getMessage()));
                }

                if (!empty($related)) {
                    // @phpstan-ignore-next-line
                    $this->cart->addProductsByIds(explode(',', $related));
                }
            }

            $this->cart->save();

            $quote->setTotalsCollectedFlag(false);
            $quote->collectTotals();
            $quote->save();

            $amount = $quote->getGrandTotal();
            $currency = $quote->getQuoteCurrencyCode();

            if (empty($currency)) {
                $currency = $quote->getStore()->getCurrentCurrency()->getCode();
            }
            
            $result = [
                'status' => 'success',
                'success' => true,
                'currency' => $currency,
                'shippingOptions' => [
                [ "id" => "free_shipping","amount" => "0", "description" => "", "label" =>"-"]
                ],
                'total' => [
                    'amount' => $this->amountProvider->convert($amount, $currency),
                ],
            ];
        } catch (\Exception $e) {
            $this->logger->debug($e->getMessage());
            $result = [
                'status' => 'error',
                'success' => false,
                'message' => $e->getMessage()
            ];
        }

        return $this->jsonHelper->jsonEncode($result);
    }

    /**
     * @return string
     */
    public function loadShippingOptions()
    {
        try {
            $request = $this->request->getBodyParams();
            $revolut_public_id = $request['revolut_public_id'];
            $address = $request['address_data'];
            $quote = $this->cart->getQuote();
            $currency = $quote->getQuoteCurrencyCode();
            $rates = [];

            if (!$quote->isVirtual()) {
                $shippingAddress = $this->reFormatAddress($address);
                $quote->getShippingAddress()->addData($shippingAddress);
                $rates = $this->shipmentEstimation->estimateByExtendedAddress($quote->getId(), $quote->getShippingAddress());
            } else {
                $currency = $quote->getQuoteCurrencyCode();
                $amount = $quote->getGrandTotal();
                $this->updateRevolutOrder($revolut_public_id, $amount, $currency, $quote);

                $shipping_options = [
                    ["id"=> "free_shipping",
                    "amount"=> "0",
                    "description"=> "",
                    "label"=> "-"
                    ]
                ];
                $result = [
                    'status' => 'success',
                    'success' => true,
                    'currency' => $currency,
                    'shippingOptions' => $shipping_options,
                    'total' => [
                        'amount' => $this->amountProvider->convert($quote->getGrandTotal(), $currency),
                    ],
                ];

                return $this->jsonHelper->jsonEncode($result);
            }

            $shouldInclTax = $this->shouldCartPriceInclTax($quote->getStore());
            $shipping_options = [];
            foreach ($rates as $rate) {
                if ($rate->getErrorMessage()) {
                    continue;
                }
                $rate_amount = $shouldInclTax ? $rate->getPriceInclTax() : $rate->getPriceExclTax();
                $shipping_options[] = [
                'id' => $rate->getCarrierCode() . '_' . $rate->getMethodCode(),
                'label' => implode(' - ', [$rate->getCarrierTitle(), $rate->getMethodTitle()]),
                'detail' => $rate->getMethodTitle(),
                'amount' =>  $this->amountProvider->convert($rate_amount, $currency)
                ];
            }
            
            if (!empty($shipping_options)) {
                $this->updateShipping($shipping_options[0]['id'], $revolut_public_id, $address);
            }

            $currency = $quote->getQuoteCurrencyCode();
            $amount = $this->cart->getQuote()->getGrandTotal();

            $result = [
                'status' => 'success',
                'success' => true,
                'currency' => $currency,
                'shippingOptions' => $shipping_options,
                'total' => [
                    'amount' => $this->amountProvider->convert($this->cart->getQuote()->getGrandTotal(), $currency),
                ],
            ];
        } catch (\Exception $e) {
            $result = [
            'status' => 'error',
            'success' => false,
            'message' => $e->getMessage()
            ];
            $this->logger->debug($e->getMessage());
        }

        return $this->jsonHelper->jsonEncode($result);
    }

    /**
     * @return string
     */
    public function setShippingOption()
    {
        $request = $this->request->getBodyParams();
        $shipping_id = $request['id_selected_carrier'];
        $revolut_public_id = $request['revolut_public_id'];
        $address = $request['address_data'];

        return $this->updateShipping($shipping_id, $revolut_public_id, $address);
    }

    public function validateOrder()
    {
        try {
            $request = $this->request->getBodyParams();

            list($billingAddress, $shippingAddress) = $this->convertAddresses($request['address']);

            $id_selected_carrier = $request['id_carrier'];
            $publicId = $request['public_id'];

            $quote = $this->cart->getQuote();
            $quote->setIsWalletButton(true);
        
            $quote->getBillingAddress()->addData($billingAddress);

            if (!$quote->isVirtual()) {
                $shipping = $quote->getShippingAddress()->addData($shippingAddress);
                $shipping->setShippingMethod($id_selected_carrier)->setCollectShippingRates(true);
            }

            $quote->setTotalsCollectedFlag(false);
            $quote->collectTotals();

            $this->storeManager->setCurrentStore($quote->getStoreId());

            if (!$this->customerSession->isLoggedIn()) {
                $quote->setCheckoutMethod(\Magento\Checkout\Model\Type\Onepage::METHOD_GUEST)
                    ->setCustomerId(0)
                    ->setCustomerEmail($quote->getBillingAddress()->getEmail())
                    ->setCustomerIsGuest(true)
                    ->setCustomerGroupId(\Magento\Customer\Api\Data\GroupInterface::NOT_LOGGED_IN_ID);
            } else {
                $quote->setCheckoutMethod(\Magento\Checkout\Model\Type\Onepage::METHOD_CUSTOMER);
            }

            $quote->getPayment()->importData(
                ['method' => \Revolut\Payment\Model\Ui\ConfigProvider::REVOLUT_PAYMENT_REQUEST_CODE,
                'additional_data' => [
                    'publicId' => $publicId
                ]]
            );

            $this->changeQuoteControl->setAllowed();
            $this->quoteRepository->save($quote);

            $result = [
                'status' => 'success',
                'success' => true,
            ];
        } catch (\Exception $e) {
            $result = [
                'status' => 'error',
                'success' => false,
                'message' => $e->getMessage()
            ];
            $this->logger->debug($e->getMessage());
        }

        return $this->jsonHelper->jsonEncode($result);
    }

    /**
     * @param string|mixed $revolut_public_id
     * @param float $amount
     * @param string $currency
     * @param object $quote
     * @return array
     */
    public function updateRevolutOrder($revolut_public_id, $amount, $currency, $quote)
    {
        $revolutOrderFactory = $this->revolutOrderFactory->create();
        $revolutOrder = $revolutOrderFactory->load($revolut_public_id, 'public_id'); // @phpstan-ignore-line

        $revolutOrderId = $revolutOrder->getRevolutOrderId();

        if (empty($revolutOrderId)) {
            throw new LocalizedException(__('Could not find Revolut Order ID'));
        }
        
        // update revolut order amount on Revolut Api
        $orderData = $this->revolutOrderApi->update($revolutOrderId, ['amount' => $this->amountProvider->convert($amount, $currency), 'currency' => $currency], $quote->getStoreId());

        if (!isset($orderData['public_id'])) {
            throw new LocalizedException(__('Can not update Revolut Order'));
        }
        $this->logger->debug('updateRevolutOrder: ' . $orderData['order_amount']['value']);

        //update revolut order amount on Magento DB
        $revolutOrderFactory->setOrderAmount($orderData['order_amount']['value']);
        $revolutOrderFactory->save();
        
        return $orderData;
    }

    /**
     * @param string $shipping_id
     * @param string $revolut_public_id
     * @param array $address
     * @return string
     */
    public function updateShipping($shipping_id, $revolut_public_id, $address)
    {
        try {
            $quote = $this->cart->getQuote();

            if (!$quote->isVirtual()) {
                $shippingAddress = $this->reFormatAddress($address);
                $shipping = $quote->getShippingAddress()->addData($shippingAddress);

                if ($shipping_id) {
                    $shipping->setShippingMethod($shipping_id)->setCollectShippingRates(true)->collectShippingRates();

                    $parts = explode('_', $shipping_id);
                    $carrierCode = array_shift($parts);
                    $methodCode = implode("_", $parts);

                    $shippingAddress = $this->inputProcessor->convertValue($shippingAddress, 'Magento\Quote\Api\Data\AddressInterface');

                    $shippingInformation = $this->shippingInformationFactory->create();
                    $shippingInformation->setShippingAddress($shippingAddress)->setShippingCarrierCode($carrierCode)->setShippingMethodCode($methodCode); // @phpstan-ignore-line
                    $this->shippingInformationManagement->saveAddressInformation($quote->getId(), $shippingInformation); // @phpstan-ignore-line

                    $quote->setTotalsCollectedFlag(false);
                    $quote->collectTotals();
                }
            }

            $currency = $quote->getQuoteCurrencyCode();
            $amount = $quote->getGrandTotal();
            $this->updateRevolutOrder($revolut_public_id, $amount, $currency, $quote);

            $result = [
                'status' => 'success',
                'success' => true,
                'currency' => $currency,
                'total' => [
                    'amount' => $this->amountProvider->convert($amount, $currency),
                ],
            ];
        } catch (\Exception $e) {
            $result = [
                'status' => 'error',
                'success' => false,
                'message' => $e->getMessage()
            ];
            $this->logger->debug($e->getMessage());
        }

        return $this->jsonHelper->jsonEncode($result);
    }

    /**
     * @param  array $address
     * @return array
     */
    public function reFormatAddress($address)
    {
        if (!is_array($address) || empty($address['country'])) {
            throw new LocalizedException(__('invalid address request'));
        }

        if (empty($address['recipient'])) {
            $firstName = null;
            $lastName = null;
        } else {
            list($firstName, $lastName) = $this->parseName($address['recipient']);
        }

        if (empty($address['region'])) {
            $regionId = null;
        } else {
            $regionId = $this->getRegionIdBy($address['region'], $address['country']);
        }

        if (empty($address['postalCode'])) {
            $address['postalCode'] = null;
        }

        if (empty($address['phone'])) {
            $address['phone'] = null;
        }

        return [
            'firstname' => $firstName,
            'lastname' => $lastName,
            'email' =>(empty($address['email']) ? "" : $address['email']),
            'street' => (empty($address['addressLine']) ? null : $address['addressLine']),
            'city' => $address['city'],
            'region_id' => $regionId,
            'region' => (empty($address['region']) ? null : $address['region']),
            'postcode' => $address['postalCode'],
            'country_id' => $address['country'],
            'telephone' => $address['phone'],
            'fax' => ''
        ];
    }

    /**
     * @param  array $address
     * @return array
     */
    public function convertAddresses($address)
    {
        $billingAddress = isset($address['billingAddress']) ? $this->reFormatAddress($address['billingAddress']) : [];
        $shippingAddress = isset($address['shippingAddress']) ? $this->reFormatAddress($address['shippingAddress']) : [];

        $billingAddress['email'] = $address['email'];
        $shippingAddress['email'] = $address['email'];

        if (empty($billingAddress['telephone']) && !empty($shippingAddress['telephone'])) {
            $billingAddress['telephone'] = $shippingAddress['telephone'];
        }
        
        if (empty($shippingAddress['telephone']) && !empty($billingAddress['telephone'])) {
            $shippingAddress['telephone'] = $billingAddress['telephone'];
        }

        return [$billingAddress, $shippingAddress];
    }

     /**
      * @param  string $name
      * @return array
      */
    public function parseName($name)
    {
        if (empty($name) || empty(trim($name))) {
            throw new LocalizedException(__("Invalid name"));
        }

        // @phpstan-ignore-next-line
        $nameParts = explode(' ', preg_replace('!\s+!', ' ', (string)trim($name)));
        // @phpstan-ignore-next-line
        if (count($nameParts) == 0) {
            throw new LocalizedException(__("Invalid name."));
        }

        $firstName = array_shift($nameParts);

        if (empty($firstName)) {
            throw new LocalizedException(__("Invalid name."));
        }

        $lastName = "";

        if ($nameParts) {
            $lastName =  array_shift($nameParts);
        }

        if (empty($lastName)) {
            $lastName = "lastname";
        }

        return [$firstName, $lastName];
    }

    /**
     * @param  string $regionName
     * @param  string $regionCountry
     * @return string|null
     */
    public function getRegionIdBy($regionName, $regionCountry)
    {
        $regions = $this->getRegionsForCountry($regionCountry);

        $regionName = strtolower(trim($regionName));

        if (isset($regions['names'][$regionName])) {
            return $regions['names'][$regionName];
        } elseif (isset($regions['codes'][$regionName])) {
            return $regions['codes'][$regionName];
        }

        return null;
    }

    /**
     * @param  string $countryCode
     * @return array
     */
    public function getRegionsForCountry($countryCode)
    {
        $country = $this->countryFactory->create()->loadByCode($countryCode);
        // @phpstan-ignore-next-line
        if (empty($country)) {
            return [];
        }

        $values = [];

        foreach ($country->getRegions() as $region) {
            $values['codes'][trim($region->getCode())] = $region->getId();
            $values['names'][trim($region->getName())] = $region->getId();
        }

        return $values;
    }

    /**
     * @param  \Magento\Store\Model\Store $store
     * @return bool
     */
    public function shouldCartPriceInclTax($store = null)
    {
        if ($this->taxHelper->displayCartBothPrices($store)) {
            return true;
        } elseif ($this->taxHelper->displayCartPriceInclTax($store)) {
            return true;
        }

        return false;
    }
}
