/*browser:true*/
/*global define*/

define([
  "ko",
  "jquery",
  "mage/storage",
  "Magento_Checkout/js/model/url-builder",
  "Magento_Customer/js/customer-data",
  window.checkoutConfig.revolutSdk,
], function (ko, $, storage, urlBuilder, customerData, RevolutCheckout) {
  "use strict";
  $(`script[src="${window.checkoutConfig.revolutSdk}"]`).attr("id", "revolut-checkout");

  return {
    isCartPage: ko.observable(false),
    publicId: ko.observable(null),
    paymentRequest: ko.observable(null),
    idSelectedCarrier: ko.observable(null),
    addressData: ko.observable(null),
    shippingOptions: [{ id: "free_shipping", amount: "0", description: "", label: "-" }],

    initPaymentRequestButton: function (isActiveLocation) {
      if (!window.checkoutConfig.productId || !isActiveLocation) {
        return false;
      }

      if (!$(`#revolut-payment-request-button`).length) {
        return false;
      }

      if (this.paymentRequest()) {
        this.paymentRequest().destroy();
      }

      const self = this;
      storage.post(urlBuilder.createUrl("/revolut/order/create", {})).done(function (response) {
        if (response && response.length) {
          response = response[0];
        }

        if (response && response.success && response.public_id) {
          self.publicId(response.public_id);
          self.createRevolutPaymentRequestButton();
        }
      });
    },

    createRevolutPaymentRequestButton: async function () {
      $("#revolut-payment-request-button").empty().append("<span></span>");

      const self = this;
      const RC = await RevolutCheckout(self.publicId());

      let paymentRequest = RC.paymentRequest({
        target: document.querySelector(`#revolut-payment-request-button span`),
        requestShipping: true,
        shippingOptions: self.shippingOptions,
        onClick() {},
        onShippingOptionChange: (selectedShippingOption) => {
          return self.setShippingOption(selectedShippingOption);
        },
        onShippingAddressChange: (selectedShippingAddress) => {
          return self.addToCart().then(() => {
            return self.loadShippingOptions(selectedShippingAddress);
          });
        },
        onSuccess: () => {
          return self.processOrder();
        },
        validate: (address) => {
          $("body").loader("show");
          return self.validateOrder(address);
        },
        onError: (error) => {
          self.displayError(error);
        },
        buttonStyle: window.checkoutConfig.themeConfigs,
      });

      paymentRequest.canMakePayment().then((result) => {
        if (result) {
          paymentRequest.render();
        } else {
          paymentRequest.destroy();
        }
      });

      this.paymentRequest(paymentRequest);
    },

    loadShippingOptions: function (address) {
      const self = this;
      return new Promise((resolve, reject) => {
        self.addressData({
          country: address.country,
          state: address.region,
          postcode: address.postalCode,
          city: address.city,
          address: "",
          address_2: "",
        });

        storage
          .post(
            urlBuilder.createUrl("/revolut/fast-checkout/shippingoptions/load", {}),
            JSON.stringify({
              address_data: self.addressData(),
              revolut_public_id: self.publicId(),
            }),
            true
          )
          .done(function (response) {
            response = JSON.parse(response);
            if (response.shippingOptions && response.shippingOptions.length > 0) {
              self.idSelectedCarrier(response.shippingOptions[0].id);
            }
            resolve(response);
          });
      });
    },

    setShippingOption: function (selected_carrier) {
      const self = this;
      return new Promise((resolve, reject) => {
        self.idSelectedCarrier(selected_carrier.id);

        storage
          .post(
            urlBuilder.createUrl("/revolut/fast-checkout/shippingoptions/set", {}),
            JSON.stringify({
              address_data: self.addressData(),
              id_selected_carrier: self.idSelectedCarrier(),
              revolut_public_id: self.publicId(),
            }),
            true
          )
          .done(function (response) {
            response = JSON.parse(response);
            resolve(response);
          });
      });
    },

    addToCart: function () {
      const self = this;
      const form = jQuery("#product_addtocart_form");

      return new Promise((resolve, reject) => {
        if (self.isCartPage()) {
          return resolve([]);
        }

        var validator = form.validation({ radioCheckboxClosest: ".nested" });

        if (!validator.valid()) {
          return self.displayError("Invalid form", true);
        }

        storage
          .post(
            urlBuilder.createUrl("/revolut/fast-checkout/addtocart", {}),
            JSON.stringify({
              request: form.serialize(),
              id_carrier: self.idSelectedCarrier(),
            }),
            true
          )
          .done(function (response) {
            response = JSON.parse(response);
            customerData.invalidate(["cart"]);
            customerData.reload(["cart"], true);
            resolve(response);
          });
      });
    },

    validateOrder: function (address) {
      const self = this;

      return new Promise((resolve, reject) => {
        storage
          .post(
            urlBuilder.createUrl("/revolut/fast-checkout/validate-order", {}),
            JSON.stringify({
              public_id: self.publicId(),
              address: address,
              id_carrier: self.idSelectedCarrier(),
            }),
            true
          )
          .done(function (response) {
            response = JSON.parse(response);

            if (response.success) {
              return resolve(response);
            }

            reject(response.message);
            self.displayError(response.message);
          });
      });
    },

    getData: function () {
      return {
        method: window.checkoutConfig.paymentRequestMethodCode,
        additional_data: {
          publicId: this.publicId(),
          setAgreement: true,
        },
      };
    },

    processOrder: function () {
      const self = this;

      return new Promise((resolve, reject) => {
        let payload = self.getData();

        storage
          .post(
            urlBuilder.createUrl("/revolut/payment-information", {}),
            JSON.stringify({ paymentMethod: payload }),
            true
          )
          .done(function (response) {
            if (response && response.length) {
              response = response[0];
            }

            if (response && response.success && response.redirectUrl) {
              var clearData = {
                selectedShippingAddress: null,
                shippingAddressFromData: null,
                newCustomerShippingAddress: null,
                selectedShippingRate: null,
                selectedPaymentMethod: null,
                selectedBillingAddress: null,
                billingAddressFromData: null,
                newCustomerBillingAddress: null,
              };

              customerData.set("checkout-data", clearData);
              customerData.invalidate(["cart"]);

              window.location.href = response.redirectUrl;
              return true;
            }

            self.cancelPayment(response.message);
          })
          .fail(function (response) {
            self.cancelPayment("An error occured while processing your order.");
          });
      });
    },

    cancelPayment: function (errorMsg) {
      const self = this;
      storage
        .post(
          urlBuilder.createUrl("/revolut/order/cancel", {}),
          JSON.stringify({
            revolut_public_id: self.publicId(),
          }),
          true
        )
        .done(function () {
          if (errorMsg) {
            self.displayError(errorMsg, true);
          }
        });
    },

    displayError: function (message, reInit = false) {
      if (reInit) {
        this.initPaymentRequestButton(reInit);
      }

      $("body").loader("hide");
      alert(message);
      console.error(message);
    },
  };
});
